use std::{cell::Cell, sync::LazyLock};

use relm4::{
    adw::subclass::prelude::ObjectSubclassType,
    gtk::{
        glib::{self, ParamSpecString, Type, object::ObjectExt, value::ToValue},
        subclass::prelude::{ObjectImpl, ObjectSubclass},
    },
};

use crate::zipline::ZiplineFolder;

glib::wrapper! {
    pub struct GtkZiplineFolder(ObjectSubclass<GtkZiplineFolderImpl>);
}

impl GtkZiplineFolder {
    pub const NONE_ID: &str = "tyroliennesupersecretnoneid";

    pub fn r#type() -> Type {
        GtkZiplineFolderImpl::type_()
    }

    pub fn none() -> Self {
        glib::Object::builder()
            .property("id", String::from(Self::NONE_ID))
            .property("name", String::from("None"))
            .build()
    }

    pub fn from_folder(folder: &ZiplineFolder) -> Self {
        glib::Object::builder()
            .property("id", folder.id.clone())
            .property("name", folder.name.clone())
            .build()
    }

    pub fn as_folder(&self) -> ZiplineFolder {
        ZiplineFolder {
            id: self.property("id"),
            name: self.property("name"),
        }
    }
}

#[derive(Default)]
pub struct GtkZiplineFolderImpl {
    id: Cell<String>,
    name: Cell<String>,
}

#[glib::object_subclass]
impl ObjectSubclass for GtkZiplineFolderImpl {
    const NAME: &str = "TyrolienneZiplineFolder";
    type Type = GtkZiplineFolder;
}

impl ObjectImpl for GtkZiplineFolderImpl {
    fn properties() -> &'static [glib::ParamSpec] {
        static PARAMS: LazyLock<[glib::ParamSpec; 2]> = LazyLock::new(|| {
            [
                ParamSpecString::builder("id").build(),
                ParamSpecString::builder("name").build(),
            ]
        });

        PARAMS.as_ref()
    }

    fn set_property(&self, _id: usize, value: &glib::Value, pspec: &glib::ParamSpec) {
        let value = value.get().unwrap();

        match pspec.name() {
            "id" => self.id.replace(value),
            "name" => self.name.replace(value),
            _ => unreachable!(),
        };
    }

    fn property(&self, _id: usize, pspec: &glib::ParamSpec) -> glib::Value {
        unsafe {
            match pspec.name() {
                "id" => (*self.id.as_ptr()).to_value(),
                "name" => (*self.name.as_ptr()).to_value(),
                _ => unreachable!(),
            }
        }
    }
}
